@@ -34,13 +34,11 @@ import io
 import json
 import re
 
-import six
-from urllib3.util import parse_url
-
 from google.auth import _helpers
 from google.auth import credentials
 from google.auth import exceptions
 from google.auth import impersonated_credentials
+from google.auth import metrics
 from google.oauth2 import sts
 from google.oauth2 import utils
 
@@ -53,12 +51,14 @@ _STS_REQUESTED_TOKEN_TYPE = "urn:ietf:params:oauth:token-type:access_token"
 # Cloud resource manager URL used to retrieve project information.
 _CLOUD_RESOURCE_MANAGER = "https://cloudresourcemanager.googleapis.com/v1/projects/"
 
+_DEFAULT_UNIVERSE_DOMAIN = "googleapis.com"
+
 
-@six.add_metaclass(abc.ABCMeta)
 class Credentials(
     credentials.Scoped,
     credentials.CredentialsWithQuotaProject,
     credentials.CredentialsWithTokenUri,
+    metaclass=abc.ABCMeta,
 ):
     """Base class for all external account credentials.
 
@@ -83,6 +83,8 @@ class Credentials(
         scopes=None,
         default_scopes=None,
         workforce_pool_user_project=None,
+        universe_domain=_DEFAULT_UNIVERSE_DOMAIN,
+        trust_boundary=None,
     ):
         """Instantiates an external account credentials object.
 
@@ -106,6 +108,9 @@ class Credentials(
                 a workload identity pool. The underlying principal must still have
                 serviceusage.services.use IAM permission to use the project for
                 billing/quota.
+            universe_domain (str): The universe domain. The default universe
+                domain is googleapis.com.
+            trust_boundary (str): String representation of trust boundary meta.
         Raises:
             google.auth.exceptions.RefreshError: If the generateAccessToken
                 endpoint returned an error.
@@ -126,14 +131,11 @@ class Credentials(
         self._scopes = scopes
         self._default_scopes = default_scopes
         self._workforce_pool_user_project = workforce_pool_user_project
-
-        Credentials.validate_token_url(token_url)
-        if token_info_url:
-            Credentials.validate_token_url(token_info_url, url_type="token info")
-        if service_account_impersonation_url:
-            Credentials.validate_service_account_impersonation_url(
-                service_account_impersonation_url
-            )
+        self._universe_domain = universe_domain or _DEFAULT_UNIVERSE_DOMAIN
+        self._trust_boundary = {
+            "locations": [],
+            "encoded_locations": "0x0",
+        }  # expose a placeholder trust boundary value.
 
         if self._client_id:
             self._client_auth = utils.ClientAuthentication(
@@ -143,6 +145,8 @@ class Credentials(
             self._client_auth = None
         self._sts_client = sts.Client(self._token_url, self._client_auth)
 
+        self._metrics_options = self._create_default_metrics_options()
+
         if self._service_account_impersonation_url:
             self._impersonated_credentials = self._initialize_impersonated_credentials()
         else:
@@ -195,6 +199,7 @@ class Credentials(
             "workforce_pool_user_project": self._workforce_pool_user_project,
             "scopes": self._scopes,
             "default_scopes": self._default_scopes,
+            "universe_domain": self._universe_domain,
         }
         if not self.is_workforce_pool:
             args.pop("workforce_pool_user_project")
@@ -286,7 +291,9 @@ class Credentials(
     def with_scopes(self, scopes, default_scopes=None):
         kwargs = self._constructor_args()
         kwargs.update(scopes=scopes, default_scopes=default_scopes)
-        return self.__class__(**kwargs)
+        scoped = self.__class__(**kwargs)
+        scoped._metrics_options = self._metrics_options
+        return scoped
 
     @abc.abstractmethod
     def retrieve_subject_token(self, request):
@@ -364,6 +371,11 @@ class Credentials(
             # is used. The client ID is sufficient for determining the user project.
             if self._workforce_pool_user_project and not self._client_id:
                 additional_options = {"userProject": self._workforce_pool_user_project}
+            additional_headers = {
+                metrics.API_CLIENT_HEADER: metrics.byoid_metrics_header(
+                    self._metrics_options
+                )
+            }
             response_data = self._sts_client.exchange_token(
                 request=request,
                 grant_type=_STS_GRANT_TYPE,
@@ -373,9 +385,17 @@ class Credentials(
                 scopes=scopes,
                 requested_token_type=_STS_REQUESTED_TOKEN_TYPE,
                 additional_options=additional_options,
+                additional_headers=additional_headers,
             )
             self.token = response_data.get("access_token")
-            lifetime = datetime.timedelta(seconds=response_data.get("expires_in"))
+            expires_in = response_data.get("expires_in")
+            # Some services do not respect the OAUTH2.0 RFC and send expires_in as a
+            # JSON String.
+            if isinstance(expires_in, str):
+                expires_in = int(expires_in)
+
+            lifetime = datetime.timedelta(seconds=expires_in)
+
             self.expiry = now + lifetime
 
     @_helpers.copy_docstring(credentials.CredentialsWithQuotaProject)
@@ -383,13 +403,25 @@ class Credentials(
         # Return copy of instance with the provided quota project ID.
         kwargs = self._constructor_args()
         kwargs.update(quota_project_id=quota_project_id)
-        return self.__class__(**kwargs)
+        new_cred = self.__class__(**kwargs)
+        new_cred._metrics_options = self._metrics_options
+        return new_cred
 
     @_helpers.copy_docstring(credentials.CredentialsWithTokenUri)
     def with_token_uri(self, token_uri):
         kwargs = self._constructor_args()
         kwargs.update(token_url=token_uri)
-        return self.__class__(**kwargs)
+        new_cred = self.__class__(**kwargs)
+        new_cred._metrics_options = self._metrics_options
+        return new_cred
+
+    @_helpers.copy_docstring(credentials.CredentialsWithUniverseDomain)
+    def with_universe_domain(self, universe_domain):
+        kwargs = self._constructor_args()
+        kwargs.update(universe_domain=universe_domain)
+        new_cred = self.__class__(**kwargs)
+        new_cred._metrics_options = self._metrics_options
+        return new_cred
 
     def _initialize_impersonated_credentials(self):
         """Generates an impersonated credentials.
@@ -413,6 +445,7 @@ class Credentials(
             service_account_impersonation_options={},
         )
         source_credentials = self.__class__(**kwargs)
+        source_credentials._metrics_options = self._metrics_options
 
         # Determine target_principal.
         target_principal = self.service_account_email
@@ -434,57 +467,18 @@ class Credentials(
             ),
         )
 
-    @staticmethod
-    def validate_token_url(token_url, url_type="token"):
-        _TOKEN_URL_PATTERNS = [
-            "^[^\\.\\s\\/\\\\]+\\.sts(?:\\.mtls)?\\.googleapis\\.com$",
-            "^sts(?:\\.mtls)?\\.googleapis\\.com$",
-            "^sts\\.[^\\.\\s\\/\\\\]+(?:\\.mtls)?\\.googleapis\\.com$",
-            "^[^\\.\\s\\/\\\\]+\\-sts(?:\\.mtls)?\\.googleapis\\.com$",
-            "^sts\\-[^\\.\\s\\/\\\\]+\\.p(?:\\.mtls)?\\.googleapis\\.com$",
-        ]
+    def _create_default_metrics_options(self):
+        metrics_options = {}
+        if self._service_account_impersonation_url:
+            metrics_options["sa-impersonation"] = "true"
+        else:
+            metrics_options["sa-impersonation"] = "false"
+        if self._service_account_impersonation_options.get("token_lifetime_seconds"):
+            metrics_options["config-lifetime"] = "true"
+        else:
+            metrics_options["config-lifetime"] = "false"
 
-        if not Credentials.is_valid_url(_TOKEN_URL_PATTERNS, token_url):
-            raise exceptions.InvalidResource(
-                "The provided {} URL is invalid.".format(url_type)
-            )
-
-    @staticmethod
-    def validate_service_account_impersonation_url(url):
-        _SERVICE_ACCOUNT_IMPERSONATION_URL_PATTERNS = [
-            "^[^\\.\\s\\/\\\\]+\\.iamcredentials\\.googleapis\\.com$",
-            "^iamcredentials\\.googleapis\\.com$",
-            "^iamcredentials\\.[^\\.\\s\\/\\\\]+\\.googleapis\\.com$",
-            "^[^\\.\\s\\/\\\\]+\\-iamcredentials\\.googleapis\\.com$",
-            "^iamcredentials\\-[^\\.\\s\\/\\\\]+\\.p\\.googleapis\\.com$",
-        ]
-
-        if not Credentials.is_valid_url(
-            _SERVICE_ACCOUNT_IMPERSONATION_URL_PATTERNS, url
-        ):
-            raise exceptions.InvalidResource(
-                "The provided service account impersonation URL is invalid."
-            )
-
-    @staticmethod
-    def is_valid_url(patterns, url):
-        """
-        Returns True if the provided URL's scheme is HTTPS and the host comforms to at least one of the provided patterns.
-        """
-        # Check specifically for whitespcaces:
-        #   Some python3.6 will parse the space character into %20 and pass the regex check which shouldn't be passed
-        if not url or len(str(url).split()) > 1:
-            return False
-
-        try:
-            uri = parse_url(url)
-        except Exception:
-            return False
-
-        if not uri.scheme or uri.scheme != "https" or not uri.hostname:
-            return False
-
-        return any(re.compile(p).match(uri.hostname.lower()) for p in patterns)
+        return metrics_options
 
     @classmethod
     def from_info(cls, info, **kwargs):
@@ -519,6 +513,7 @@ class Credentials(
             credential_source=info.get("credential_source"),
             quota_project_id=info.get("quota_project_id"),
             workforce_pool_user_project=info.get("workforce_pool_user_project"),
+            universe_domain=info.get("universe_domain", _DEFAULT_UNIVERSE_DOMAIN),
             **kwargs
         )
 
